@@ -38,26 +38,6 @@ class Command(BaseCommand):
             default=DEFAULT_DIR_PATH,
         )
 
-        # parser.add_argument(
-        #     "-f",
-        #     "--force",
-        #     action="store_true",
-        #     help="Skips sanity checks and imports all items",
-        # ) will be added in follow up
-
-    @staticmethod
-    def po_keys_sanity_check(obj_keys):
-        """
-        This method returns a QuerySet of already imported translations
-        """
-        key_ids = []
-        for key in obj_keys:
-            key_ids.append(key.id_name)
-
-        already_existing_keys = TranslationKey.objects.filter(id_name__in=key_ids)
-        log.info(f"Already existing po keys: {len(already_existing_keys)}")
-        return already_existing_keys
-
     @staticmethod
     def read_po_files(dirs) -> dict:
         """Read po files."""
@@ -76,10 +56,13 @@ class Command(BaseCommand):
         return files_with_languages
 
     @staticmethod
-    def create_po_keys(file_path, file_entries):
+    def create_po_keys(file_entries):
         """Create po keys."""
-        entries = [
-            TranslationKey(
+
+        existing_po_keys = {key.id_name: key for key in TranslationKey.objects.all()}
+
+        new_po_keys = {
+            entry.msgid: TranslationKey(
                 id_name=entry.msgid,
                 id_name_plural=entry.msgid_plural,
                 encoding=entry.encoding,
@@ -88,21 +71,23 @@ class Command(BaseCommand):
                 flags=entry.flags,
             )
             for entry in file_entries
-        ]
+            if entry.msgid not in existing_po_keys.keys()
+        }
 
-        return entries
+        return existing_po_keys, new_po_keys
 
     @staticmethod
-    def create_po_translations(file_path, file_entries, key_entries, language):
-        """Import po translations."""
+    def create_po_translations(file_entries, key_entries, language) -> list:
+        """Create po translations."""
+
         entries = [
             Translation(
                 language=language,
-                key=key_entries[idx],
+                key=key_entries.get(entry.msgid),
                 translation=entry.msgstr if entry.msgstr else entry.msgid,
                 translation_plural=entry.msgid_plural,
             )
-            for idx, entry in enumerate(file_entries)
+            for entry in file_entries
         ]
 
         return entries
@@ -112,72 +97,51 @@ class Command(BaseCommand):
         """Process po files."""
         po_files = Command.read_po_files(dirs)
 
-        raw_key_entries, translations = [], []
+        existing_keys, new_key_entries = {}, {}
+        for lang, files in sorted(po_files.items()):
+            for file_path, file_entries in files:
+                d1, d2 = Command.create_po_keys(file_entries)
+                existing_keys = {**existing_keys, **d1}
+                new_key_entries = {**new_key_entries, **d2}
+
+        log.info(f"Found {len(existing_keys.items())} already existing TranslationKeys.")
+        log.info(f"Prepared {len(new_key_entries.items())} of new TranslationKeys via po files.")
+
+        inserts = TranslationKey.objects.bulk_create(new_key_entries.values(), ignore_conflicts=True)
+        if not inserts:
+            log.info("NO NEW PO TRANSLATIONS HAVE BEEN ADDED")
+        else:
+            log.info(f"Done. Inserted {len(inserts)} TranslationKeys from po files.")
+        log.info("----------------------------------------------------")
+
+        all_keys = {**existing_keys, **new_key_entries}
+
+        translations = defaultdict(list)
         for lang, files in sorted(po_files.items()):
             language, _ = Language.objects.get_or_create(lang_info=lang)
             for file_path, file_entries in files:
-                raw_key_entries += Command.create_po_keys(file_path, file_entries)
-                translations += Command.create_po_translations(file_path, file_entries, raw_key_entries, language)
+                translations[lang].extend(Command.create_po_translations(file_entries, all_keys, language))
 
-        log.info(f"Prepared {len(raw_key_entries)} of RAW TranslationKeys via po files.")
-        log.info("Cleaning...")
-        uniq = {}
-        duplicate_counter = 0
-        for key in raw_key_entries:
-            if key.id_name in uniq.keys():
-                duplicate_counter += 1
-            else:
-                uniq[key.id_name] = key
-
-        log.info(f"Duplicate ID_NAME occurred with {duplicate_counter} TranslationKeys.")
-
-        bulk_inserts = uniq
-        already_imported_keys = Command.po_keys_sanity_check(uniq.values())
-
-        if not already_imported_keys:
-            log.info("NO ALREADY IMPORTED KEYS FOUND!")
-        else:
-            for key in already_imported_keys:
-                if key.id_name in uniq.keys():
-                    bulk_inserts.pop(key.id_name)
-
-        inserts = TranslationKey.objects.bulk_create(bulk_inserts.values(), ignore_conflicts=True)
-        if not inserts:
-            log.info("NO NEW PO TRANSLATIONS HAVE BEEN ADDED")
-        log.info(f"Done. Inserted {len(inserts)} po TranslationKeys.")
-        log.info("----------------------------------------------------")
-
-        log.info(f"Prepared {len(translations)} of RAW Translations via po files.")
-
-        translation_inserts = []
-        uniq_id_names = set(uniq.keys())  # uniq dictionary keys are id_names
-
-        log.info("Cleaning!")
-        for translation_obj in translations:
-            if translation_obj.translation in uniq_id_names:
-                translation_inserts.append(translation_obj)
-
-        # translation_inserts = translations this will be used later for force argument
-
-        integrity_errors = 0
-        try:
-            log.info(f"Prepared {len(translation_inserts)} of Translations. Inserting bulk... ")
-            translation_inserts = Translation.objects.bulk_create(translation_inserts, ignore_conflicts=True)
-        except django.db.utils.IntegrityError:
-            log.info("Integrity error occurred, and bulk create was canceled! Inserting individually...")
+        for lang, translations_list in translations.items():
             translation_inserts = []
-            for obj in translations:
-                if obj.translation in uniq.keys():
+            integrity_errors = 0
+            try:
+                log.info(f"Prepared {len(translations_list)} Translations for {lang} language. Inserting bulk... ")
+                translation_inserts = Translation.objects.bulk_create(translations_list, ignore_conflicts=True)
+            except django.db.utils.IntegrityError:
+                log.info("Integrity error occurred, and bulk create was canceled! Inserting individually...")
+                for obj in translations_list:
                     try:
                         obj.save()
                         translation_inserts.append(obj)
                     except django.db.utils.IntegrityError:
                         integrity_errors += 1
 
-        log.info(f"Integrity error occurred with {integrity_errors} Translations.")
-        if not translation_inserts:
-            log.info("NO NEW PO TRANSLATIONS HAVE BEEN ADDED")
-        log.info(f"Done. Inserted {len(translation_inserts)} po Translations.")
+            log.info(f"Integrity error occurred with {integrity_errors} Translations for {lang} language.")
+            if not translation_inserts:
+                log.info("NO NEW PO TRANSLATIONS HAVE BEEN ADDED")
+            log.info(f"Done. Inserted {len(translation_inserts)} Translations from po files for {lang} language.")
+            log.info("----------------------------------------------------")
 
     @staticmethod
     def create_json_keys(language_data, language_code):
@@ -282,7 +246,6 @@ class Command(BaseCommand):
 
     def handle(self, *args, **options):
         """Entrypoint to the command."""
-        # force = options.get("force")
 
         dir_name = Path(settings.BASE_DIR.parent, options.get("translations_dir"))
 
